#! /usr/bin/env node


/* eslint-disable no-console */

'use strict';

var fileExists = require('file-exists');
var path = require('path');
var chalk = require('chalk');
var minimist = require('minimist');
var Table = require('cli-table2');
var _ = require('lodash');

var atomicS3 = require('./main');
var prepareOptions = require('./prepare-options');
var validateOptions = require('./validate-options');

var args = minimist(process.argv.slice(2));
var configFile = args.config || 'atomic-s3.config.js';
var hrstart = process.hrtime();

// merge options from possible config file with command line arguments
var opts = {};
if (fileExists(configFile)) {
  var resolvedPath = path.resolve(configFile);
  if (args.verbose) {
    console.log('Using config from ' + chalk.blue(resolvedPath));
  }
  opts = require(resolvedPath);
} else if (args.config) {
  console.log('[Error] ' + configFile + ' does not exist');
  process.exit(1);
}

if (!opts.s3options) {
  opts.s3options = {};
}
if (!opts.s3options.params) {
  opts.s3options.params = {};
}

if (args.bucket) {
  opts.s3options.params.Bucket = args.bucket;
}
if (args.region) {
  opts.s3options.region = args.region;
}
if (args.path) {
  opts.path = args.path;
}

// check that options are valid
opts = prepareOptions(opts);
var errors = validateOptions(opts);
if (errors) {
  console.log(chalk.red('Error: invalid options'));
  _.forEach(errors, function (msg) {
    console.log('  ' + msg);
  });
  process.exit(1);
}

if (!args.silent) {
  console.log(chalk.cyan('▶') + ' Uploading to S3');
}

if (args.verbose) {
  var table = new Table();
  table.push({ path: opts.path });
  table.push({ bucket: opts.s3options.params.Bucket });
  table.push({ region: opts.s3options.region });
  table.push({ entryPoints: opts.entryPoints.join(',') });
  console.log(table.toString());
}

atomicS3.publish(opts, function (err) {
  if (err) {
    console.log(chalk.red('[Error]') + ' ' + err);
    process.exit(1);
  }
  if (!args.silent) {
    console.log(chalk.green('✓') + ' ' // eslint-disable-line prefer-template
     + 'Finished S3 upload in ' + chalk.magenta((process.hrtime(hrstart)[1] / 1000000).toFixed(2) + ' ms'));
  }
});