'use strict';

var awspublish = require('gulp-awspublish');
var del = require('del');
var vfs = require('vinyl-fs');
var path = require('path');
var through2 = require('through2').obj;
var stream = require('stream');

var prepareOptions = require('./prepare-options');
var validateOptions = require('./validate-options');

/*
 * Publish the given source files to AWS
 * with the given headers
 */
function publishToS3(publisher, simulate, force) {
  if (force) {
    del.sync('./.awspublish-*');
  }

  var bucket = publisher.config.params.Bucket;

  // Config object is passed to
  // new AWS.S3() as documented here:
  //   http://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/S3.html#constructor-property

  // We want to construct a pipeline that consists of multiple segments piped together.
  // The consumer of this pipeline needs to be able to pipe into its first segment
  // and to maybe continue it, i.e. to access its last segment. So we're returning
  // an array with the first and last segments of this pipeline.

  var first = publisher.publish({ 'x-amz-acl': 'private' }, {
    force: force,
    simulate: simulate === true
  });
  var cache = null;
  if (!force) {
    cache = first.pipe(publisher.cache());
  }
  var reporter = awspublish.reporter();
  if (simulate === true) {
    reporter = through2(function (file, _enc, cb) {
      console.log('s3://' + bucket + '/' + file.s3.path); // eslint-disable-line
      console.log(file.s3.headers); // eslint-disable-line
      cb(null, file);
    });
  }
  var last = (cache || first).pipe(reporter);
  return [first, last];
}

//
// https://github.com/jussi-kalliokoski/gulp-awspublish-router/blob/master/lib/utils/initFile.js
//
function s3Init(file, s3Folder) {
  if (file.s3) {
    return;
  }
  file.s3 = {}; // eslint-disable-line no-param-reassign
  file.s3.headers = {}; // eslint-disable-line no-param-reassign
  file.s3.path = file.path // eslint-disable-line no-param-reassign
  .replace(file.base, s3Folder || '').replace(new RegExp('\\' + path.sep + 'g'), '/');
}

/*
 * Get file streams for all entry points assets
 * (assets without rev urls)
 */
function entryPointStream(sourceFolder, entryPoints, s3Folder) {
  return vfs.src(entryPoints, {
    cwd: sourceFolder || 'dist'
  }).pipe(through2(function (file, _enc, cb) {
    s3Init(file, s3Folder);
    cb(null, file);
  }));
}

/*
 * Get file streams for all hashed assets
 * (assets with rev urls)
 *
 * targetFolder -- folder to publish into
 * maxAge -- expiry age for header
 */
function assetStream(sourceFolder, entryPoints, maxAge, s3Folder) {
  if (maxAge === null || !isFinite(maxAge)) {
    maxAge = 3600; // eslint-disable-line no-param-reassign
  }

  var headers = {
    'Cache-Control': 'max-age=' + maxAge + ', public'
  };

  // Select everything BUT the entrypoints
  var src = entryPoints.map(function (f) {
    return '!' + f;
  });
  src.unshift('**/*.*');

  return vfs.src(src, {
    cwd: sourceFolder || 'dist'
  }).pipe(through2(function (file, _enc, cb) {
    s3Init(file, s3Folder);
    Object.assign(file.s3.headers, headers);
    cb(null, file);
  }));
}

function publishInSeries(streams, opts) {
  // We want to construct a new stream that combines others
  // sequentially. We pipe to it the first one, passing the option end: false,
  // listen for the 'end' event of the first stream and then pipe it the second one,
  // not passing the end option.

  var output = stream.PassThrough({ // eslint-disable-line new-cap
    objectMode: true
  });

  var _loop = function (i) {
    var nextStream = streams[i + 1];
    streams[i].once('end', function () {
      return nextStream.pipe(output);
    });
  };

  for (var i = 0; i < streams.length - 1; i++) {
    _loop(i);
  }

  var publisher = awspublish.create(opts.s3options);
  var s3 = publishToS3(publisher, opts.simulateDeployment || false, opts.forceDeployment || false); // we get the first and last segments of the pipeline

  streams[0].pipe(output, {
    end: false
  }).pipe(s3[0]);

  return s3[1];
}

function publish(options, cb) {
  var opts = prepareOptions(options);
  var errors = validateOptions(opts);
  if (errors) {
    throw Error(errors);
  }

  var asset = assetStream(opts.path, opts.entryPoints, opts.maxAge, '');
  var entry = entryPointStream(opts.path, opts.entryPoints);

  // It is important to do deploy in series to
  // achieve an "atomic" update. uploading index.html
  // before hashed assets would be bad -- JOJ

  publishInSeries([asset, entry], opts).on('end', function () {
    cb(false);
  }).on('error', function (err) {
    cb(err);
  });
}

module.exports = {
  entryPointStream: entryPointStream,
  assetStream: assetStream,
  publishToS3: publishToS3,
  publishInSeries: publishInSeries,
  publish: publish
};